/*
 * Copyright (c) 2015-2016, Sumio Morioka
 * All rights reserved.
 *
 * This source code was originally written by Dr.Sumio Morioka for use in the 2015-2016 issues of 
 * "the Interface magazine", published by CQ publishing Co.Ltd in Japan (http://www.cqpub.co.jp).
 * The author has no responsibility on any results caused by using this code.
 *
 * - Author: Dr.Sumio Morioka (http://www002.upp.so-net.ne.jp/morioka, FB:Sumio Morioka)
 *
 *
 * IMPORTANT NOTICE:
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the copyright holder nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <time.h>

#include "cv.h"
#include "highgui.h"
//using namespace cv;		// comment in if preferable

#include "opencv_func.h"
#include "websocket_flags.h"

//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// any functions/global variables
//////////////////////////////////////////////////////////////////////////// <<

void opencv_func_init(void *)
{
	cv::namedWindow(WINNAME, CV_WINDOW_AUTOSIZE);

//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// preparation for image processing
//////////////////////////////////////////////////////////////////////////// <<
}


void opencv_func_run(IMG_PROCESSING_PARAM *param)
{
//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// image processing main part
	{
		int src_x	= (*(param->srcImage)).cols;	// camera frame
		int src_y	= (*(param->srcImage)).rows;	// camera frame
		int dst_x	= (*(param->dstImage)).cols;	// frame for transfer
		int dst_y	= (*(param->dstImage)).rows;	// frame for transfer

		time_t		t;
		struct tm	*t_st;
		char strbuf[128];

		time(&t);
		t_st	= localtime(&t);

		sprintf(strbuf, "%d/%d/%d %02d:%02d:%02d", 
						t_st->tm_year + 1900, t_st->tm_mon + 1, t_st->tm_mday,
						t_st->tm_hour, t_st->tm_min, t_st->tm_sec);

		if (zoom_in == 0) {
			// shrink entire image
			cv::resize(*(param->srcImage), *(param->dstImage), (*(param->dstImage)).size(), CV_INTER_NN);
		}
		else {
			int zx	= zoom_x;		// (clicked pos) save volatile variable
			int zy	= zoom_y;		// (clicked pos) save volatile variable

			// adjust cut position
			zx	= zx - (dst_x / 2);			// move cut position so that clicked pos comes to center
			if (zx < 0)						// underflow
				zx	= 0;
			if (zx + dst_x >= src_x)		// overflow
				zx	= src_x - dst_x;

			zy	= zy - (dst_y / 2);			// move cut position so that clicked pos comes to center
			if (zy < 0)						// underflow
				zy	= 0;
			if (zy + dst_y >= src_y)		// overflow
				zy	= src_y - dst_y;

			// cut image
			cv::Mat src_img	= *(param->srcImage);
			cv::Mat cut_img	= src_img(cv::Rect(zx, zy, dst_x, dst_y));
			cut_img.copyTo(*(param->dstImage));
		}

		// rotate 180 degree (if necessary)
//		cv::Mat	rotmat;
//		rotmat.create(cv::Size(3, 2), CV_32FC1);
//		rotmat	= cv::getRotationMatrix2D(cv::Point(dst_x / 2, dst_y / 2), 180, 1);				// center, angle, scale
//		cv::warpAffine(*(param->dstImage), *(param->dstImage), rotmat, cv::Size(dst_x, dst_y));	// rotate

		// write time
		cv::putText(*(param->dstImage), strbuf, cv::Point(20,20), cv::FONT_HERSHEY_PLAIN, 1.0, cv::Scalar(0,0,0), 1, CV_AA);
	}
//////////////////////////////////////////////////////////////////////////// <<
}


void opencv_func_stop(void *)
{
	cv::destroyWindow(WINNAME);

//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// release resources for program termination
//////////////////////////////////////////////////////////////////////////// <<
}

// end of file
