/*
 * Copyright (c) 2015-2016, Sumio Morioka
 * All rights reserved.
 *
 * This source code was originally written by Dr.Sumio Morioka for use in the 2015-2016 issues of 
 * "the Interface magazine", published by CQ publishing Co.Ltd in Japan (http://www.cqpub.co.jp).
 * The author has no responsibility on any results caused by using this code.
 *
 * - Author: Dr.Sumio Morioka (http://www002.upp.so-net.ne.jp/morioka, FB:Sumio Morioka)
 *
 *
 * IMPORTANT NOTICE:
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the copyright holder nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <time.h>

#include "cv.h"
#include "highgui.h"
//using namespace cv;		// comment in if preferable

#include "opencv_func.h"
#include "websocket_flags.h"

//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// any functions/global variables
//////////////////////////////////////////////////////////////////////////// <<

void opencv_func_init(void *)
{
	cv::namedWindow(WINNAME, CV_WINDOW_AUTOSIZE);

//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// preparation for image processing
//////////////////////////////////////////////////////////////////////////// <<
}


void opencv_func_run(IMG_PROCESSING_PARAM *param)
{
//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// image processing main part
	{
		int dst_x	= (*(param->dstImage)).cols;	// display size
		int dst_y	= (*(param->dstImage)).rows;	// display size

		cv::Mat	panoImage, segmentImage, map_matrix;

		panoImage.create(cv::Size(PANO_SEG_W * pano_total_segments_g, PANO_SEG_H), CV_8UC3);
		segmentImage.create(cv::Size(PANO_SEG_W, PANO_SEG_H), CV_8UC3);

		// process 360-deg panorama image
		int	x_pos, seg, deg;

		for (seg = 0, x_pos = 0, deg = 0; seg < pano_total_segments_g; seg++, deg += (int)(PANO_INC_DEG), x_pos += PANO_SEG_W) {
			int			xx[4], yy[4];
			cv::Point2f	src_pnt[4], dst_pnt[4];

			// determine location of a window segment on round image
			xx[0]	= (int)(PANO_CENTER_X + PANO_INNER_R * cos(deg2rad((float)deg)));	// LU
			yy[0]	= (int)(PANO_CENTER_Y - PANO_INNER_R * sin(deg2rad((float)deg)));	// LU

			xx[1]	= (int)(PANO_CENTER_X + PANO_INNER_R * cos(deg2rad((float)deg + (float)(PANO_INC_DEG))));	// RU
			yy[1]	= (int)(PANO_CENTER_Y - PANO_INNER_R * sin(deg2rad((float)deg + (float)(PANO_INC_DEG))));	// RU

			xx[2]	= (int)(PANO_CENTER_X + PANO_OUTER_R * cos(deg2rad((float)deg + (float)(PANO_INC_DEG))));	// RD
			yy[2]	= (int)(PANO_CENTER_Y - PANO_OUTER_R * sin(deg2rad((float)deg + (float)(PANO_INC_DEG))));	// RD

			xx[3]	= (int)(PANO_CENTER_X + PANO_OUTER_R * cos(deg2rad((float)deg)));	// LD
			yy[3]	= (int)(PANO_CENTER_Y - PANO_OUTER_R * sin(deg2rad((float)deg)));	// LD

			for (int i = 0; i < 4; i++) {
				src_pnt[i] = cvPoint2D32f(xx[i], yy[i]);
			}

	  		// specify corresponding coordinates on flat image
	  		dst_pnt[0] = cvPoint2D32f(0.0, 0.0);									// LU
			dst_pnt[1] = cvPoint2D32f((float)(PANO_SEG_W), 0.0);					// RU
	 		dst_pnt[2] = cvPoint2D32f((float)(PANO_SEG_W), (float)(PANO_SEG_H));	// RD
			dst_pnt[3] = cvPoint2D32f(0.0, (float)(PANO_SEG_H));					// LD

			// convert round segment image to flat image
			map_matrix	= cv::getPerspectiveTransform(src_pnt, dst_pnt);
			cv::warpPerspective(*(param->srcImage), segmentImage, map_matrix, cv::Size(segmentImage.cols, segmentImage.rows));

			// merge the segment to output image
			{
				cv::Mat substImage;		// place here
				substImage	= panoImage(cv::Rect(x_pos, 0, PANO_SEG_W, PANO_SEG_H));
				segmentImage.copyTo(substImage);
			}
  		}

		// create output image
		if ((pano_disp_window_g + 1) * pano_window_width_g < PANO_SEG_W * pano_total_segments_g) {	// no overflow
			// cut from panorama image
			cv::Mat cut_img	= panoImage(cv::Rect(pano_disp_window_g * pano_window_width_g, 0, pano_window_width_g, PANO_SEG_H));
			// scale size
			cv::Mat	ds;
			ds.create(cv::Size(dst_x, dst_y), CV_8UC3);
			cv::resize(cut_img, ds, ds.size(), CV_INTER_NN);
			// copy image
			memcpy((*(param->dstImage)).data, ds.data, dst_x * dst_y * 3);
			ds.release();
		}
//		else {	// partial overflow
//		}

		// rotate 180 degree (if necessary)
//		cv::Mat	rotmat;
//		rotmat.create(cv::Size(3, 2), CV_32FC1);
//		rotmat	= cv::getRotationMatrix2D(cv::Point(dst_x / 2, dst_y / 2), 180, 1);				// center, angle, scale
//		cv::warpAffine(*(param->dstImage), *(param->dstImage), rotmat, cv::Size(dst_x, dst_y));	// rotate

		panoImage.release();
		segmentImage.release();


		///////////////////////////////////////////////////////////////////////
		// optional: write time
		///////////////////////////////////////////////////////////////////////
		time_t		t;
		struct tm	*t_st;
		char strbuf[128];

		time(&t);
		t_st	= localtime(&t);

		sprintf(strbuf, "%d/%d/%d %02d:%02d:%02d", 
						t_st->tm_year + 1900, t_st->tm_mon + 1, t_st->tm_mday,
						t_st->tm_hour, t_st->tm_min, t_st->tm_sec);

		cv::putText(*(param->dstImage), strbuf, cv::Point(20,20), cv::FONT_HERSHEY_PLAIN, 1.0, cv::Scalar(0,255,0), 1, CV_AA);

		///////////////////////////////////////////////////////////////////////
		// optional: write gauge
		///////////////////////////////////////////////////////////////////////
		int	gauge_start, gauge_end;

		gauge_start	= (pano_disp_window_g * 360) / pano_total_window_g;
		gauge_end	= ((pano_disp_window_g + 1) * 360) / pano_total_window_g;
		if (gauge_end > 360)
			gauge_end	= 360;

		cv::ellipse(*(param->dstImage), cv::Point(50,70), cv::Size(30,30), 0, 0, 360, CV_RGB(0, 0, 255), 3, 0);
		cv::ellipse(*(param->dstImage), cv::Point(50,70), cv::Size(30,30), 0, gauge_start, gauge_end, CV_RGB(255, 0, 0), 3, 0);
	}
//////////////////////////////////////////////////////////////////////////// <<
}


void opencv_func_stop(void *)
{
	cv::destroyWindow(WINNAME);

//////////////////////////////////////////////////////////////////////////// >>
// ADD YOUR ORIGINAL CODE HERE
// release resources for program termination
//////////////////////////////////////////////////////////////////////////// <<
}

// end of file
